@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1999-2000 Thomas Roessler <roessler@does-not-exist.org>
+ * Copyright (C) 1999-2001 Thomas Roessler <roessler@does-not-exist.org>
  * 
  *     This program is free software; you can redistribute it
  *     and/or modify it under the terms of the GNU General Public
@@ -535,8 +535,7 @@ void mix_make_chain (LIST **chainp, int *redraw)
   
   mix_screen_coordinates (type2_list, &coords, chain, 0);
   
-  menu = mutt_new_menu ();
-  menu->menu = MENU_MIX;
+  menu = mutt_new_menu (MENU_MIX);
   menu->max = ttll;
   menu->make_entry = mix_entry;
   menu->tag = NULL;
