@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
+ * Copyright (C) 1996-2000,2007 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -99,7 +99,7 @@ status_format_str (char *buf, size_t buflen, size_t col, char op, const char *sr
       if (Context && Context->path)
       {
 	strfcpy (tmp, Context->path, sizeof (tmp));
-	mutt_pretty_mailbox (tmp);
+	mutt_pretty_mailbox (tmp, sizeof (tmp));
       }
       else
 	strfcpy (tmp, _("(no mailbox)"), sizeof (tmp));
